@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 IBM Corporation
 # Copyright 2015-2017 Lenovo
 #
@@ -14,15 +12,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# This represents the low layer message framing portion of IPMI
+
+"""This represents the low layer message framing portion of IPMI"""
 
 from itertools import chain
+import os
+import socket
+import struct
+import threading
+
 import pyghmi.constants as const
 import pyghmi.exceptions as exc
-
 import pyghmi.ipmi.events as sel
 import pyghmi.ipmi.fru as fru
+import pyghmi.ipmi.oem.generic as genericoem
 from pyghmi.ipmi.oem.lookup import get_oem_handler
+import pyghmi.ipmi.private.util as pygutil
+from pyghmi.ipmi import sdr
 
 try:
     from pyghmi.ipmi.private import session
@@ -32,10 +38,6 @@ try:
     from pyghmi.ipmi.private import localsession
 except ImportError:
     localsession = None
-import pyghmi.ipmi.private.util as pygutil
-import pyghmi.ipmi.sdr as sdr
-import socket
-import struct
 
 try:
     range = xrange
@@ -83,6 +85,11 @@ power_states = {
 }
 
 
+def select_simplesession():
+    global session
+    import pyghmi.ipmi.private.simplesession as session
+
+
 def _mask_to_cidr(mask):
     maskn = struct.unpack_from('>I', mask)[0]
     cidr = 32
@@ -96,6 +103,21 @@ def _cidr_to_mask(prefix):
     return struct.pack('>I', 2 ** prefix - 1 << (32 - prefix))
 
 
+class Housekeeper(threading.Thread):
+    """A Maintenance thread for housekeeping
+
+    Long lived use of pyghmi may warrant some recurring asynchronous behavior.
+    This stock thread provides a simple minimal context for these housekeeping
+    tasks to run in.  To use, do 'pyghmi.ipmi.command.Maintenance().start()'
+    and from that point forward, pyghmi should execute any needed ongoing
+    tasks automatically as needed.  This is an alternative to calling
+    wait_for_rsp or eventloop in a thread of the callers design.
+    """
+
+    def run(self):
+        Command.eventloop()
+
+
 class Command(object):
     """Send IPMI commands to BMCs.
 
@@ -116,19 +138,27 @@ class Command(object):
     :param onlogon: function to run when logon completes in an asynchronous
                     fashion.  This will result in a greenthread behavior.
     :param kg: Optional parameter to use if BMC has a particular Kg configured
+    :param verifycallback: For OEM extensions that use HTTPS, this function
+                           will be used to evaluate the certificate.
+    :param keepalive: If False, then an idle connection will logout rather
+                      than keepalive unless held open by console or ongoing
+                      activity.
     """
 
     def __init__(self, bmc=None, userid=None, password=None, port=623,
-                 onlogon=None, kg=None):
+                 onlogon=None, kg=None, privlevel=None, verifycallback=None,
+                 keepalive=True):
         # TODO(jbjohnso): accept tuples and lists of each parameter for mass
         # operations without pushing the async complexities up the stack
         self.onlogon = onlogon
         self.bmc = bmc
+        self._sdrcachedir = None
         self._sdr = None
         self._oem = None
+        self._oemknown = False
         self._netchannel = None
         self._ipv6support = None
-        self.certverify = None
+        self.certverify = verifycallback
         if bmc is None:
             self.ipmi_session = localsession.Session()
         elif onlogon is not None:
@@ -137,7 +167,9 @@ class Command(object):
                                                 password=password,
                                                 onlogon=self.logged,
                                                 port=port,
-                                                kg=kg)
+                                                kg=kg,
+                                                privlevel=privlevel,
+                                                keepalive=keepalive)
             # induce one iteration of the loop, now that we would be
             # prepared for it in theory
             session.Session.wait_for_rsp(0)
@@ -146,7 +178,19 @@ class Command(object):
                                                 userid=userid,
                                                 password=password,
                                                 port=port,
-                                                kg=kg)
+                                                kg=kg,
+                                                privlevel=privlevel)
+
+    def set_sdr_cachedir(self, path):
+        """Register use of a directory for SDR cache.
+
+        Takes the given directory and uses it to persist SDR cache run to run.
+        This can greatly improve performance across runs.
+
+        :param path:
+        :return:
+        """
+        self._sdrcachedir = path
 
     def register_key_handler(self, callback, type='tls'):
         """Assign a verification handler for a public key
@@ -170,8 +214,8 @@ class Command(object):
 
     @classmethod
     def eventloop(cls):
-        while session.Session.wait_for_rsp():
-            pass
+        while True:
+            session.Session.wait_for_rsp()
 
     @classmethod
     def wait_for_rsp(cls, timeout):
@@ -204,9 +248,14 @@ class Command(object):
         method does an interrogation to identify the OEM.
 
         """
-        if self._oem:
+        if self._oemknown:
             return
-        self._oem = get_oem_handler(self._get_device_id(), self)
+        if self.bmc is None:
+            self._oem = genericoem.OEMHandler(None, None)
+            self._oemknown = True
+            return
+        self._oem, self._oemknown = get_oem_handler(self._get_device_id(),
+                                                    self)
 
     def get_bootdev(self):
         """Get current boot device override information.
@@ -225,12 +274,12 @@ class Command(object):
             raise exc.IpmiException(response['error'])
         # this should only be invoked for get system boot option complying to
         # ipmi spec and targeting the 'boot flags' parameter
-        assert (response['command'] == 9 and
-                response['netfn'] == 1 and
-                response['data'][0] == 1 and
-                (response['data'][1] & 0b1111111) == 5)
-        if (response['data'][1] & 0b10000000 or
-                not response['data'][2] & 0b10000000):
+        assert (response['command'] == 9
+                and response['netfn'] == 1
+                and response['data'][0] == 1
+                and (response['data'][1] & 0b1111111) == 5)
+        if (response['data'][1] & 0b10000000
+                or not response['data'][2] & 0b10000000):
             return {'bootdev': 'default', 'persistent': True}
         else:  # will consult data2 of the boot flags parameter for the data
             persistent = False
@@ -301,8 +350,7 @@ class Command(object):
         waitattempts = 300
         if not isinstance(wait, bool):
             waitattempts = wait
-        if (wait and
-           newpowerstate in ('on', 'off', 'shutdown', 'softoff')):
+        if wait and newpowerstate in ('on', 'off', 'shutdown', 'softoff'):
             if newpowerstate in ('softoff', 'shutdown'):
                 waitpowerstate = 'off'
             else:
@@ -332,8 +380,8 @@ class Command(object):
         return self._oem.get_video_launchdata()
 
     def reset_bmc(self):
-        """Do a cold reset in BMC
-        """
+        """Do a cold reset in BMC"""
+
         response = self.raw_command(netfn=6, command=2)
         if 'error' in response:
             raise exc.IpmiException(response['error'])
@@ -386,7 +434,7 @@ class Command(object):
         return {'bootdev': bootdev}
 
     def xraw_command(self, netfn, command, bridge_request=(), data=(),
-                     delay_xmit=None, retry=True, timeout=None):
+                     delay_xmit=None, retry=True, timeout=None, rslun=0):
         """Send raw ipmi command to BMC, raising exception on error
 
         This is identical to raw_command, except it raises exceptions
@@ -409,14 +457,33 @@ class Command(object):
         rsp = self.ipmi_session.raw_command(netfn=netfn, command=command,
                                             bridge_request=bridge_request,
                                             data=data, delay_xmit=delay_xmit,
-                                            retry=retry, timeout=timeout)
+                                            retry=retry, timeout=timeout,
+                                            rslun=rslun)
         if 'error' in rsp:
             raise exc.IpmiException(rsp['error'], rsp['code'])
-        rsp['data'] = buffer(bytearray(rsp['data']))
+        rsp['data'] = buffer(rsp['data'])
         return rsp
 
+    def get_diagnostic_data(self, savefile, progress=None, autosuffix=False):
+        if os.path.exists(savefile) and not os.path.isdir(savefile):
+            raise exc.InvalidParameterValue(
+                'Not allowed to overwrite existing file: {0}'.format(
+                    savefile))
+        self.oem_init()
+        return self._oem.get_diagnostic_data(savefile, progress, autosuffix)
+
+    def get_description(self):
+        """Get physical attributes for the system, e.g. for GUI use
+
+        :returns: dict -- dict containing attributes, 'height' is for
+               how many U tall, 'slot' for what slot in a blade enclosure
+               or 0 if not blade, for example.
+        """
+        self.oem_init()
+        return self._oem.get_description()
+
     def raw_command(self, netfn, command, bridge_request=(), data=(),
-                    delay_xmit=None, retry=True, timeout=None):
+                    delay_xmit=None, retry=True, timeout=None, rslun=0):
         """Send raw ipmi command to BMC
 
         This allows arbitrary IPMI bytes to be issued.  This is commonly used
@@ -434,10 +501,12 @@ class Command(object):
         :param timeout: A custom amount of time to wait for initial reply
         :returns: dict -- The response from IPMI device
         """
-        return self.ipmi_session.raw_command(netfn=netfn, command=command,
-                                             bridge_request=bridge_request,
-                                             data=data, delay_xmit=delay_xmit,
-                                             retry=retry, timeout=timeout)
+        rsp = self.ipmi_session.raw_command(netfn=netfn, command=command,
+                                            bridge_request=bridge_request,
+                                            data=data, delay_xmit=delay_xmit,
+                                            retry=retry, timeout=timeout,
+                                            rslun=rslun)
+        return rsp
 
     def get_power(self):
         """Get current power state of the managed system
@@ -506,7 +575,7 @@ class Command(object):
         # sdr timestamp, our data version revision, aux firmware revision,
         # and oem defined field
         if self._sdr is None:
-            self._sdr = sdr.SDR(self)
+            self._sdr = sdr.SDR(self, self._sdrcachedir)
         return self._sdr
 
     def get_event_log(self, clear=False):
@@ -553,9 +622,8 @@ class Command(object):
         returns an iterable of string descriptions
         """
         yield "System"
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
-        for fruid in self._sdr.fru:
+        self.init_sdr()
+        for fruid in sorted(self._sdr.fru):
             yield self._sdr.fru[fruid].fru_name
         self.oem_init()
         for compname in self._oem.get_oem_inventory_descriptions():
@@ -570,12 +638,12 @@ class Command(object):
         self.oem_init()
         if component == 'System':
             return self._get_zero_fru()
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
+        self.init_sdr()
         for fruid in self._sdr.fru:
             if self._sdr.fru[fruid].fru_name == component:
                 return self._oem.process_fru(fru.FRU(
-                    ipmicmd=self, fruid=fruid, sdr=self._sdr.fru[fruid]).info)
+                    ipmicmd=self, fruid=fruid,
+                    sdr=self._sdr.fru[fruid]).info, component)
         return self._oem.get_inventory_of_component(component)
 
     def _get_zero_fru(self):
@@ -614,13 +682,13 @@ class Command(object):
         """
         self.oem_init()
         yield ("System", self._get_zero_fru())
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
-        for fruid in self._sdr.fru:
+        self.init_sdr()
+        for fruid in sorted(self._sdr.fru):
             fruinf = fru.FRU(
                 ipmicmd=self, fruid=fruid, sdr=self._sdr.fru[fruid]).info
             if fruinf is not None:
-                fruinf = self._oem.process_fru(fruinf)
+                fruinf = self._oem.process_fru(fruinf,
+                                               self._sdr.fru[fruid].fru_name)
             yield (self._sdr.fru[fruid].fru_name, fruinf)
         for componentpair in self._oem.get_oem_inventory():
             yield componentpair
@@ -657,10 +725,18 @@ class Command(object):
         warning, critical, or failed assessments.
         """
         summary = {'badreadings': [], 'health': const.Health.Ok}
-        for reading in self.get_sensor_data():
-            if reading.health != const.Health.Ok:
-                summary['health'] |= reading.health
-                summary['badreadings'].append(reading)
+        fallbackreadings = []
+        try:
+            self.oem_init()
+            fallbackreadings = self._oem.get_health(summary)
+            for reading in self.get_sensor_data():
+                if reading.health != const.Health.Ok:
+                    summary['health'] |= reading.health
+                    summary['badreadings'].append(reading)
+        except exc.BypassGenericBehavior:
+            pass
+        if not summary['badreadings']:
+            summary['badreadings'] = fallbackreadings
         return summary
 
     def get_sensor_reading(self, sensorname):
@@ -672,11 +748,13 @@ class Command(object):
         :param sensorname:  Name of the desired sensor
         :returns: sdr.SensorReading object
         """
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
+        self.init_sdr()
         for sensor in self._sdr.get_sensor_numbers():
             if self._sdr.sensors[sensor].name == sensorname:
-                rsp = self.raw_command(command=0x2d, netfn=4, data=(sensor,))
+                currsensor = self._sdr.sensors[sensor]
+                rsp = self.raw_command(command=0x2d, netfn=4,
+                                       rslun=currsensor.sensor_lun,
+                                       data=(currsensor.sensor_number,))
                 if 'error' in rsp:
                     raise exc.IpmiException(rsp['error'], rsp['code'])
                 return self._sdr.sensors[sensor].decode_sensor_reading(
@@ -694,7 +772,7 @@ class Command(object):
         fetchcmd = bytearray((channel, param, 0, 0))
         fetched = self.xraw_command(0xc, 2, data=fetchcmd)
         fetchdata = fetched['data']
-        if ord(fetchdata[0]) != 17:
+        if bytearray(fetchdata)[0] != 17:
             return None
         if len(fetchdata) == 5:  # IPv4 address
             if prefixlen:
@@ -711,10 +789,42 @@ class Command(object):
                 return None
             return mac
         elif len(fetchdata) == 2:
-            return ord(fetchdata[1])
+            return bytearray(fetchdata)[1]
         else:
             raise Exception("Unrecognized data format " + repr(fetchdata))
 
+    def get_extended_bmc_configuration(self):
+        self.oem_init()
+        return self._oem.get_extended_bmc_configuration()
+
+    def get_bmc_configuration(self):
+        self.oem_init()
+        return self._oem.get_bmc_configuration()
+
+    def set_bmc_configuration(self, changeset):
+        self.oem_init()
+        return self._oem.set_bmc_configuration(changeset)
+
+    def clear_bmc_configuration(self):
+        self.oem_init()
+        return self._oem.clear_bmc_configuration()
+
+    def get_system_configuration(self, hideadvanced=True):
+        self.oem_init()
+        return self._oem.get_system_configuration(hideadvanced)
+
+    def set_system_configuration(self, changeset):
+        self.oem_init()
+        return self._oem.set_system_configuration(changeset)
+
+    def clear_system_configuration(self):
+        """Clear the Bios/UEFI configuration
+
+        This requests the system revert to factory default settings
+        """
+        self.oem_init()
+        self._oem.clear_system_configuration()
+
     def set_net_configuration(self, ipv4_address=None, ipv4_configuration=None,
                               ipv4_gateway=None, channel=None):
         """Set network configuration data.
@@ -854,10 +964,12 @@ class Command(object):
 
         :returns: Iterator of sdr.SensorReading objects
         """
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
+        self.init_sdr()
         for sensor in self._sdr.get_sensor_numbers():
-            rsp = self.raw_command(command=0x2d, netfn=4, data=(sensor,))
+            currsensor = self._sdr.sensors[sensor]
+            rsp = self.raw_command(command=0x2d, netfn=4,
+                                   rslun=currsensor.sensor_lun,
+                                   data=(currsensor.sensor_number,))
             if 'error' in rsp:
                 if rsp['code'] == 203:  # Sensor does not exist, optional dev
                     continue
@@ -874,8 +986,7 @@ class Command(object):
 
         :returns: Iterator of dicts describing each sensor
         """
-        if self._sdr is None:
-            self._sdr = sdr.SDR(self)
+        self.init_sdr()
         for sensor in self._sdr.get_sensor_numbers():
             yield {'name': self._sdr.sensors[sensor].name,
                    'type': self._sdr.sensors[sensor].sensor_type}
@@ -904,7 +1015,8 @@ class Command(object):
                         continue
                     else:
                         raise
-                chantype = ord(rsp['data'][1]) & 0b1111111
+                chantype = bytearray(rsp['data'])[1]
+                chantype = chantype & 0b1111111
                 if chantype in (4, 6):
                     try:
                         # Some implementations denote an inactive channel
@@ -918,9 +1030,10 @@ class Command(object):
                         # However some implementations may still have
                         # ambiguous channel info, that will need to be
                         # picked up on an OEM extension...
-                        self._netchannel = ord(rsp['data'][0]) & 0b1111
+                        netchan = bytearray(rsp['data'])[0]
+                        self._netchannel = netchan & 0b1111
                         break
-                    except exc.IpmiException as ie:
+                    except exc.IpmiException:
                         # This means the attempt to fetch parameter 5 failed,
                         # therefore move on to next candidate channel
                         continue
@@ -935,7 +1048,7 @@ class Command(object):
             channel = self.get_network_channel()
         rqdata = (channel, 0x11, 0, 0)
         rsp = self.xraw_command(netfn=0xc, command=2, data=rqdata)
-        return ord(rsp['data'][1])
+        return bytearray(rsp['data'])[1]
 
     def get_alert_destination(self, destination=0, channel=None):
         """Get alert destination
@@ -967,10 +1080,10 @@ class Command(object):
         destinfo['retries'] = retries
         rqdata = (channel, 19, destination, 0)
         rsp = self.xraw_command(netfn=0xc, command=2, data=rqdata)
-        if ord(rsp['data'][2]) & 0b11110000 == 0:
+        if bytearray(rsp['data'])[2] & 0b11110000 == 0:
             destinfo['address_format'] = 'ipv4'
             destinfo['address'] = socket.inet_ntoa(rsp['data'][4:8])
-        elif ord(rsp['data'][2]) & 0b11110000 == 0b10000:
+        elif bytearray(rsp['data'])[2] & 0b11110000 == 0b10000:
             destinfo['address_format'] = 'ipv6'
             destinfo['address'] = socket.inet_ntop(socket.AF_INET6,
                                                    rsp['data'][3:])
@@ -1016,7 +1129,7 @@ class Command(object):
         # of entries.  We have no guarantee that the meaningful data will
         # be contiguous
         rsp = self.xraw_command(netfn=4, command=0x13, data=(8, 0, 0))
-        numpol = ord(rsp['data'][1])
+        numpol = bytearray(rsp['data'])[1]
         desiredchandest = (channel << 4) | destination
         availpolnum = None
         for polnum in range(1, numpol + 1):
@@ -1107,8 +1220,9 @@ class Command(object):
                                                          channel)
             if destdata:
                 self.xraw_command(netfn=0xc, command=1, data=destdata)
-        if (acknowledge_required is not None or retries is not None or
-                acknowledge_timeout is not None):
+        if (acknowledge_required is not None
+                or retries is not None
+                or acknowledge_timeout is not None):
             currtype = self.xraw_command(netfn=0xc, command=2, data=(
                 channel, 18, destination, 0))
             if currtype['data'][0] != b'\x11':
@@ -1129,6 +1243,23 @@ class Command(object):
         if not ip == '0.0.0.0':
             self._assure_alert_policy(channel, destination)
 
+    def get_hostname(self):
+        """Get the hostname used by the BMC in various contexts
+
+        This can vary somewhat in interpretation, but generally speaking
+        this should be the name that shows up on UIs and in DHCP requests and
+        DNS registration requests, as applicable.
+
+        :return: current hostname
+        """
+        self.oem_init()
+        try:
+            return self._oem.get_hostname()
+        except exc.UnsupportedFunctionality:
+            # Use the DCMI MCI field as a fallback, since it's the closest
+            # thing in the IPMI Spec for this
+            return self.get_mci()
+
     def get_mci(self):
         """Get the Management Controller Identifier, per DCMI specification
 
@@ -1136,6 +1267,20 @@ class Command(object):
         """
         return self._chunkwise_dcmi_fetch(9)
 
+    def set_hostname(self, hostname):
+        """Set the hostname to be used by the BMC in various contexts.
+
+        See get_hostname for details
+
+        :param hostname: The hostname to set
+        :return: Nothing
+        """
+        self.oem_init()
+        try:
+            return self._oem.set_hostname(hostname)
+        except exc.UnsupportedFunctionality:
+            return self.set_mci(hostname)
+
     def set_mci(self, mci):
         """Set the management controller identifier, per DCMI specification
 
@@ -1158,10 +1303,10 @@ class Command(object):
     def _chunkwise_dcmi_fetch(self, command):
         szdata = self.xraw_command(
             netfn=0x2c, command=command, data=(0xdc, 0, 0))
-        totalsize = ord(szdata['data'][1])
+        totalsize = bytearray(szdata['data'])[1]
         chksize = 0xf
         offset = 0
-        retstr = ''
+        retstr = b''
         while offset < totalsize:
             if (offset + chksize) > totalsize:
                 chksize = totalsize - offset
@@ -1169,6 +1314,8 @@ class Command(object):
                 netfn=0x2c, command=command, data=(0xdc, offset, chksize))
             retstr += chk['data'][2:]
             offset += chksize
+        if not isinstance(retstr, str):
+            retstr = retstr.decode('utf-8')
         return retstr
 
     def _chunkwise_dcmi_set(self, command, data):
@@ -1491,6 +1638,9 @@ class Command(object):
             'proprietary': 5,
             'no_access': 0x0F,
         }
+        self.oem_init()
+        self._oem.set_user_access(
+            uid, channel, callback, link_auth, ipmi_msg, privilege_level)
         data = [b, uid & 0b00111111,
                 privilege_levels[privilege_level] & 0b00001111, 0]
         response = self.raw_command(netfn=0x06, command=0x43, data=data)
@@ -1553,10 +1703,13 @@ class Command(object):
         :param name: username (limit of 16bytes)
         """
         data = [uid]
+        if not isinstance(name, bytes):
+            name = name.encode('utf-8')
         if len(name) > 16:
             raise Exception('name must be less than or = 16 chars')
-        name = name.ljust(16, "\x00")
-        data.extend([ord(x) for x in name])
+        name = name.ljust(16, b'\x00')
+        name = bytearray(name)
+        data.extend(name)
         self.xraw_command(netfn=0x06, command=0x45, data=data)
         return True
 
@@ -1658,13 +1811,17 @@ class Command(object):
                 ipmi_msg (bool)
                 privilege_level: (str)[callback, user, operatorm administrator,
                                        proprietary, no_access]
+            expiration:
+                None for 'unknown', 0 for no expiry, days to expire otherwise.
         """
         if channel is None:
             channel = self.get_network_channel()
         name = self.get_user_name(uid)
         access = self.get_user_access(uid, channel)
+        self.oem_init()
+        expiration = self._oem.get_user_expiration(uid)
         data = {'name': name, 'uid': uid, 'channel': channel,
-                'access': access['access']}
+                'access': access['access'], 'expiration': expiration}
         return data
 
     def get_name_uids(self, name, channel=None):
@@ -1729,11 +1886,11 @@ class Command(object):
         if channel is None:
             channel = self.get_network_channel()
         self.set_user_name(uid, name)
+        self.set_user_password(uid, password=password)
+        self.set_user_password(uid, mode='enable', password=password)
         self.set_user_access(uid, channel, callback=callback,
                              link_auth=link_auth, ipmi_msg=ipmi_msg,
                              privilege_level=privilege_level)
-        self.set_user_password(uid, password=password)
-        self.set_user_password(uid, mode='enable', password=password)
         return True
 
     def user_delete(self, uid, channel=None):
@@ -1782,14 +1939,14 @@ class Command(object):
         self.set_user_password(uid, mode)
         return True
 
-    def get_firmware(self):
-        """Retrieve OEM Firmware information
-        """
+    def get_firmware(self, components=()):
+        """Retrieve OEM Firmware information"""
+
         self.oem_init()
         mcinfo = self.xraw_command(netfn=6, command=1)
-        bmcver = '{0}.{1}'.format(
-            ord(mcinfo['data'][2]), hex(ord(mcinfo['data'][3]))[2:])
-        return self._oem.get_oem_firmware(bmcver)
+        major, minor = struct.unpack('BB', mcinfo['data'][2:4])
+        bmcver = '{0}.{1}'.format(major, hex(minor)[2:])
+        return self._oem.get_oem_firmware(bmcver, components)
 
     def get_capping_enabled(self):
         """Get PSU based power capping status
@@ -1808,14 +1965,14 @@ class Command(object):
         return self._oem.set_oem_capping_enabled(enable)
 
     def get_remote_kvm_available(self):
-        """Get remote KVM availability
-        """
+        """Get remote KVM availability"""
+
         self.oem_init()
         return self._oem.get_oem_remote_kvm_available()
 
     def get_domain_name(self):
-        """Get Domain name
-        """
+        """Get Domain name"""
+
         self.oem_init()
         return self._oem.get_oem_domain_name()
 
@@ -1832,20 +1989,21 @@ class Command(object):
         self.oem_init()
         return self._oem.get_graphical_console()
 
-    def update_firmware(self, file, data=None, progress=None, bank=None):
+    def update_firmware(self, filename, data=None, progress=None, bank=None):
         """Send file to BMC to perform firmware update
 
-         :param filename:  The filename to upload to the target BMC
-         :param data:  The payload of the firmware.  Default is to read from
-                       specified filename.
-         :param progress:  A callback that will be given a dict describing
-                           update process.  Provide if
+         :param filename: The filename to upload to the target BMC
+         :param data: The payload of the firmware.  Default is to read from
+                      specified filename.
+         :param progress: A callback that will be given a dict describing
+                          update process.  Provide if
          :param bank: Indicate a target 'bank' of firmware if supported
         """
+
         self.oem_init()
         if progress is None:
             progress = lambda x: True
-        return self._oem.update_firmware(file, data, progress, bank)
+        return self._oem.update_firmware(filename, data, progress, bank)
 
     def attach_remote_media(self, url, username=None, password=None):
         """Attach remote media by url
@@ -1875,7 +2033,7 @@ class Command(object):
 
         :param filename: The filename to use, the basename of the parameter
                          will be given to the bmc.
-        :param filename: Optional callback for progress updates
+        :param progress: Optional callback for progress updates
         """
         self.oem_init()
         return self._oem.upload_media(filename, progress)
@@ -1887,3 +2045,23 @@ class Command(object):
         """
         self.oem_init()
         return self._oem.list_media()
+
+    def get_licenses(self):
+        self.oem_init()
+        return self._oem.get_licenses()
+
+    def delete_license(self, name):
+        self.oem_init()
+        return self._oem.delete_license(name)
+
+    def save_licenses(self, directory):
+        if os.path.exists(directory) and not os.path.isdir(directory):
+            raise exc.InvalidParameterValue(
+                'Not allowed to overwrite existing file: {0}'.format(
+                    directory))
+        self.oem_init()
+        return self._oem.save_licenses(directory)
+
+    def apply_license(self, filename, progress=None):
+        self.oem_init()
+        return self._oem.apply_license(filename, progress)
