@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2017 Lenovo
+# Copyright 2017-2019 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,21 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# from Matthew Garret's 'firmware_config' project.
+"""from Matthew Garret's 'firmware_config' project.
 
-# This contains functions to manage the firmware configuration of Lenovo
-# servers
+This contains functions to manage the firmware configuration of Lenovo servers
+"""
 
 import ast
-import struct
 import random
-import pyghmi.exceptions as pygexc
+import struct
 
+import pyghmi.exceptions as pygexc
 from pyghmi.ipmi.oem.lenovo import EfiDecompressor
+import six
 
 try:
-    from lxml import etree
     import EfiCompressor
+    from lxml import etree
 except ImportError:
     etree = None
     EfiCompressor = None
@@ -73,8 +72,8 @@ class _ExpEngine(object):
         for optkey in self.cfg:
             opt = self.cfg[optkey]
             lid = opt['lenovo_id'].replace('-', '_')
-            if (lid == category and
-                    opt['lenovo_setting'] == setting):
+            if (lid == category
+                    and opt['lenovo_setting'] == setting):
                 self.relatedsettings.add(optkey)
                 return opt['lenovo_value']
         return None
@@ -134,11 +133,12 @@ class LenovoFirmwareConfig(object):
         self.connection = ipmicmd
 
     def imm_size(self, filename):
-        data = []
-        data += LENOVO_ENTERPRISE
-        data += SIZE_COMMAND
-        for i in range(len(filename)):
-            data += [ord(filename[i])]
+        data = bytearray()
+        data.extend(LENOVO_ENTERPRISE)
+        data.extend(SIZE_COMMAND)
+        if not isinstance(filename, bytes):
+            filename = filename.encode('utf-8')
+        data.extend(filename)
 
         response = run_command_with_retry(self.connection, data=data)
 
@@ -150,24 +150,24 @@ class LenovoFirmwareConfig(object):
     def imm_open(self, filename, write=False, size=None):
         response = None
         retries = 12
-        data = []
-        data += LENOVO_ENTERPRISE
+        data = bytearray()
+        data.extend(LENOVO_ENTERPRISE)
         if write is False:
-            data += OPEN_RO_COMMAND
+            data.extend(OPEN_RO_COMMAND)
         else:
             assert size is not None
-            data += OPEN_WO_COMMAND
+            data.extend(OPEN_WO_COMMAND)
             hex_size = struct.pack("<I", size)
-            for byte in hex_size[:4]:
-                data += [ord(byte)]
-            data += [0x01, 0x10]
-        for i in range(len(filename)):
-            data += [ord(filename[i])]
+            data.extend(bytearray(hex_size[:4]))
+            data.extend([0x01, 0x10])
+        if not isinstance(filename, bytes):
+            filename = filename.encode('utf-8')
+        data.extend(filename)
         while len(data) < 38:
-            data += [0x00]
+            data.append(0)
 
         while retries:
-            retries = retries-1
+            retries = retries - 1
             response = run_command_with_retry(self.connection, data=data)
             try:
                 if response['code'] == 0 or retries == 0:
@@ -185,10 +185,7 @@ class LenovoFirmwareConfig(object):
         data += CLOSE_COMMAND
 
         hex_filehandle = struct.pack("<I", filehandle)
-
-        for byte in hex_filehandle[:4]:
-            data += [ord(byte)]
-
+        data.extend(bytearray(hex_filehandle[:4]))
         run_command_with_retry(self.connection, data=data)
 
     def imm_write(self, filehandle, size, inputdata):
@@ -199,20 +196,17 @@ class LenovoFirmwareConfig(object):
         hex_filehandle = struct.pack("<I", filehandle)
 
         while remaining > 0:
-            data = []
-            data += LENOVO_ENTERPRISE
-            data += WRITE_COMMAND
-            for byte in hex_filehandle[:4]:
-                data += [ord(byte)]
+            data = bytearray()
+            data.extend(LENOVO_ENTERPRISE)
+            data.extend(WRITE_COMMAND)
+            data.extend(hex_filehandle[:4])
             hex_offset = struct.pack("<I", offset)
-            for byte in hex_offset[:4]:
-                data += [ord(byte)]
+            data.extend(hex_offset[:4])
             if remaining < blocksize:
                 amount = remaining
             else:
                 amount = blocksize
-            for byte in inputdata[offset:offset+amount]:
-                data += [ord(byte)]
+            data.extend(inputdata[offset:offset + amount])
             remaining -= blocksize
             offset += blocksize
             run_command_with_retry(self.connection, data=data)
@@ -220,7 +214,7 @@ class LenovoFirmwareConfig(object):
     def imm_read(self, filehandle, size):
         blocksize = 0xc8
         offset = 0
-        output = ''
+        output = b''
         remaining = size
 
         hex_filehandle = struct.pack("<I", filehandle)
@@ -230,15 +224,12 @@ class LenovoFirmwareConfig(object):
             data = []
             data += LENOVO_ENTERPRISE
             data += READ_COMMAND
-            for byte in hex_filehandle[:4]:
-                data += [ord(byte)]
+            data.extend(bytearray(hex_filehandle[:4]))
             hex_offset = struct.pack("<I", offset)
-            for byte in hex_offset[:4]:
-                data += [ord(byte)]
+            data.extend(bytearray(hex_offset[:4]))
             if remaining < blocksize:
                 hex_blocksize = struct.pack("<H", remaining)
-            for byte in hex_blocksize[:2]:
-                data += [ord(byte)]
+            data.extend(hex_blocksize[:2])
             remaining -= blocksize
             offset += blocksize
 
@@ -256,12 +247,16 @@ class LenovoFirmwareConfig(object):
                 options[option]['new_value'] = options[option]['default']
         self.set_fw_options(options)
 
-    def get_fw_options(self):
+    def get_fw_options(self, fetchimm=True):
+        if fetchimm:
+            cfgfilename = "config.efi"
+        else:
+            cfgfilename = "config"
         options = {}
         data = None
-        for i in range(0, 30):
-            filehandle = self.imm_open("config.efi")
-            size = self.imm_size("config.efi")
+        for _ in range(0, 30):
+            filehandle = self.imm_open(cfgfilename)
+            size = self.imm_size(cfgfilename)
             data = self.imm_read(filehandle, size)
             self.imm_close(filehandle)
             data = EfiDecompressor.decompress(data)
@@ -289,6 +284,7 @@ class LenovoFirmwareConfig(object):
                 else:
                     lenovo_group = group.get("ID")
                 for setting in group.iter("setting"):
+                    forceinstance = False
                     is_list = False
                     lenovo_setting = setting.get("ID")
                     protect = True if setting.get("protected") == 'true' \
@@ -306,36 +302,28 @@ class LenovoFirmwareConfig(object):
                     name = setting.find("mriName").text
                     help = setting.find("desc").text
                     onedata = setting.find('text_data')
-                    if onedata:
-                        if len(list(onedata.iter('instance'))) > 1:
-                            protect = True
                     if onedata is not None:
                         if onedata.get('password') == 'true':
                             protect = True
                     enumdata = setting.find('enumerate_data')
                     if enumdata is not None:
                         if enumdata.get('maxinstance') is not None:
-                            protect = True
-                        for currchoice in enumdata.iter('choice'):
-                            if len(list(currchoice.iter('instance'))) > 1:
-                                protect = True
-                                break
+                            forceinstance = True
                     if onedata is None:
                         onedata = setting.find('numeric_data')
                     if onedata is not None:
                         if onedata.get('maxinstance') is not None:
-                            protect = True  # Not yet supported
+                            forceinstance = True
+                        instances = list(onedata.iter('instance'))
+                        if not instances:
+                            protect = True  # not supported yet
                         else:
-                            instance = onedata.find('instance')
-                            if instance is None:
-                                protect = True  # not supported yet
-                            else:
-                                current = instance.text
-                            default = onedata.get('default', None)
-                            if default == '':
-                                default = None
-                    if (setting.find('cmd_data') is not None or
-                            setting.find('boolean_data') is not None):
+                            current = [x.text for x in instances]
+                        default = onedata.get('default', None)
+                        if default == '':
+                            default = None
+                    if (setting.find('cmd_data') is not None
+                            or setting.find('boolean_data') is not None):
                         protect = True  # Hide currently unsupported settings
                     ldata = setting.find("list_data")
                     extraorder = False
@@ -346,11 +334,11 @@ class LenovoFirmwareConfig(object):
                         current = []
                         extraorder = ldata.get('ordered') == 'true'
                     lenovo_value = None
+                    instancetochoicemap = {}
                     for choice in setting.iter("choice"):
                         label = choice.find("label").text
                         possible.append(label)
-                        instance = choice.find("instance")
-                        if instance is not None:
+                        for instance in choice.iter("instance"):
                             if is_list:
                                 if not extraorder:
                                     current.append(label)
@@ -358,7 +346,11 @@ class LenovoFirmwareConfig(object):
                                     currentdict[
                                         int(instance.get("order"))] = label
                             else:
-                                current = label
+                                currid = instance.get('ID')
+                                if currid:
+                                    instancetochoicemap[currid] = label
+                                else:
+                                    current = label
                                 try:
                                     lenovo_value = int(
                                         choice.find('value').text)
@@ -384,6 +376,64 @@ class LenovoFirmwareConfig(object):
                             default.append(currentdef[order])
                     optionname = "%s.%s" % (cfglabel, name)
                     alias = "%s.%s" % (lenovo_id, name)
+                    if onedata is not None:
+                        if current and len(current) > 1:
+                            instidx = 1
+                            for inst in current:
+                                optname = '{0}.{1}'.format(optionname, instidx)
+                                options[optname] = dict(
+                                    current=inst,
+                                    default=default,
+                                    possible=possible,
+                                    pending=None,
+                                    new_value=None,
+                                    help=help,
+                                    is_list=is_list,
+                                    lenovo_value=lenovo_value,
+                                    lenovo_id=lenovo_id,
+                                    lenovo_group=lenovo_group,
+                                    lenovo_setting=lenovo_setting,
+                                    lenovo_reboot=reset,
+                                    lenovo_protect=protect,
+                                    lenovo_instance=instidx,
+                                    readonly_expression=readonly,
+                                    hide_expression=hide,
+                                    sortid=sortid,
+                                    alias=alias)
+                                sortid += 1
+                                instidx += 1
+                            continue
+                        if current:
+                            current = current[0]
+                    if instancetochoicemap:
+                        for currid in sorted(instancetochoicemap):
+                            optname = '{0}.{1}'.format(optionname, currid)
+                            current = instancetochoicemap[currid]
+                            options[optname] = dict(
+                                current=current,
+                                default=default,
+                                possible=possible,
+                                pending=None,
+                                new_value=None,
+                                help=help,
+                                is_list=is_list,
+                                lenovo_value=lenovo_value,
+                                lenovo_id=lenovo_id,
+                                lenovo_group=lenovo_group,
+                                lenovo_setting=lenovo_setting,
+                                lenovo_reboot=reset,
+                                lenovo_protect=protect,
+                                lenovo_instance=currid,
+                                readonly_expression=readonly,
+                                hide_expression=hide,
+                                sortid=sortid,
+                                alias=alias)
+                            sortid += 1
+                        continue
+                    lenovoinstance = ""
+                    if forceinstance:
+                        optionname = '{0}.{1}'.format(optionname, 1)
+                        lenovoinstance = 1
                     options[optionname] = dict(current=current,
                                                default=default,
                                                possible=possible,
@@ -397,10 +447,10 @@ class LenovoFirmwareConfig(object):
                                                lenovo_setting=lenovo_setting,
                                                lenovo_reboot=reset,
                                                lenovo_protect=protect,
+                                               lenovo_instance=lenovoinstance,
                                                readonly_expression=readonly,
                                                hide_expression=hide,
                                                sortid=sortid,
-                                               lenovo_instance="",
                                                alias=alias)
                     sortid = sortid + 1
         for opt in options:
@@ -436,8 +486,7 @@ class LenovoFirmwareConfig(object):
                         ','.join(sorted(options[option]['readonly_why'])))
                     errstr += ea
                 raise pygexc.InvalidParameterValue(errstr)
-            if (isinstance(options[option]['new_value'], str) or
-                    isinstance(options[option]['new_value'], unicode)):
+            if isinstance(options[option]['new_value'], six.string_types):
                 # Coerce a simple string parameter to the expected list format
                 options[option]['new_value'] = [options[option]['new_value']]
             options[option]['pending'] = options[option]['new_value']
@@ -450,7 +499,8 @@ class LenovoFirmwareConfig(object):
             setting = etree.Element('setting',
                                     ID=options[option]['lenovo_setting'])
             if options[option]['lenovo_group'] is not None:
-                group = etree.Element('group', ID=options[option]['lenovo_group'])
+                group = etree.Element('group',
+                                      ID=options[option]['lenovo_group'])
                 config.append(group)
                 group.append(setting)
             else:
@@ -471,11 +521,11 @@ class LenovoFirmwareConfig(object):
                 if is_list:
                     count += 1
                     instance = etree.Element(
-                        'instance', ID=options[option]['lenovo_instance'],
+                        'instance', ID=str(options[option]['lenovo_instance']),
                         order=str(count))
                 else:
                     instance = etree.Element(
-                        'instance', ID=options[option]['lenovo_instance'])
+                        'instance', ID=str(options[option]['lenovo_instance']))
                 choice.append(instance)
 
         if not changes:
