@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2016-2017 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,23 +12,44 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pyghmi.constants as pygconst
-import pyghmi.exceptions as pygexc
-import pyghmi.ipmi.private.session as ipmisession
-import pyghmi.ipmi.sdr as sdr
-import pyghmi.util.webclient as webclient
+import fnmatch
 import struct
-import urllib
 import weakref
 from xml.etree.ElementTree import fromstring
 import zipfile
 
+import pyghmi.constants as pygconst
+import pyghmi.exceptions as pygexc
+import pyghmi.ipmi.private.session as ipmisession
+from pyghmi.ipmi import sdr
+import pyghmi.util.webclient as webclient
+import six
+
+try:
+    from urllib import urlencode
+except ImportError:
+    from urllib.parse import urlencode
+
 try:
     range = xrange
 except NameError:
     pass
 
 
+def stringtoboolean(originput, name):
+    input = originput.lower()
+    try:
+        num = int(input)
+    except ValueError:
+        num = None
+    if 'enabled'.startswith(input) or 'yes'.startswith(input) or num == 1:
+        return True
+    elif 'disabled'.startswith(input) or 'no'.startswith(input) or num == 0:
+        return False
+    raise pygexc.InvalidParameterValue('{0} is an invalid setting for '
+                                       '{1}'.format(originput, name))
+
+
 def fpc_read_ac_input(ipmicmd):
     rsp = ipmicmd.xraw_command(netfn=0x32, command=0x90, data=(1,))
     rsp = rsp['data']
@@ -100,7 +119,7 @@ def fpc_get_nodeperm(ipmicmd, number, sz):
         rsp['data'] = b'\x00' + bytes(rsp['data'])
     elif len(rsp['data']) == 6:  # New FPC format
         rsp['data'] = rsp['data'][:2] + rsp['data'][3:]
-    perminfo = ord(rsp['data'][1])
+    perminfo = bytearray(rsp['data'])[1]
     if sz == 6:  # FPC
         permfail = ('\x02', '\x03')
     elif sz == 2:  # SMM
@@ -251,6 +270,180 @@ class SMMClient(object):
         self.password = ipmicmd.ipmi_session.password
         self._wc = None
 
+    def clear_bmc_configuration(self):
+        self.ipmicmd.xraw_command(0x32, 0xad)
+
+    rulemap = {
+        'password_reuse_count': 'passwordReuseCheckNum',
+        'password_change_interval': 'passwordChangeInterval',
+        'password_expiration': 'passwordDurationDays',
+        'password_login_failures': 'passwordFailAllowdNum',
+        'password_min_length': 'passwordMinLength',
+        'password_lockout_period': 'passwordLockoutTimePeriod',
+    }
+
+    fanmodes = {
+        1: 'Capped_20%',
+        2: 'Capped_25%',
+        3: 'Capped_30%',
+        4: 'Capped_45%',
+        0: 'Normal',
+        5: 'Boosted',
+    }
+
+    def get_bmc_configuration(self):
+        settings = {}
+        self.wc.request(
+            'POST', '/data',
+            ('get=passwordMinLength,passwordForceChange,passwordDurationDays,'
+             'passwordExpireWarningDays,passwordChangeInterval,'
+             'passwordReuseCheckNum,passwordFailAllowdNum,'
+             'passwordLockoutTimePeriod'))
+        rsp = self.wc.getresponse()
+        rspbody = rsp.read()
+        accountinfo = fromstring(rspbody)
+        for rule in self.rulemap:
+            settings[rule] = {'value': int(
+                accountinfo.find(self.rulemap[rule]).text)}
+        rsp = self.ipmicmd.xraw_command(0x34, 3)
+        fanmode = self.fanmodes[bytearray(rsp['data'])[0]]
+        settings['fanspeed'] = {
+            'value': fanmode, 'default': 'Normal',
+            'help': ('Adjust the fan speed of the D2 Chassis. Capped settings '
+                     'will reduce fan speed for better acoustic experience at '
+                     'the expense of performance.  Normal is using the Lenovo '
+                     'engineered cooling adjustments across the full range. '
+                     'Boosted adds fanspeed to the Normal response to '
+                     'provide more aggressive cooling.'),
+            'possible': [self.fanmodes[x] for x in self.fanmodes]}
+        powercfg = self.ipmicmd.xraw_command(0x32, 0xa2)
+        powercfg = bytearray(powercfg['data'])
+        val = powercfg[0]
+        if val == 1:
+            val = 'N+1'
+        elif val == 0:
+            val = 'Disable'
+        settings['power_redundancy'] = {
+            'default': 'N+1',
+            'value': val,
+            'possible': ['N+1', 'Disable'],
+            'help': ('Configures allowed power budget according to expected '
+                     'redundancy. If N+1, power caps will be set to keep '
+                     'servers from using more power than the installed power '
+                     'supplies could supply if one fails.  If disabled, '
+                     'power budget is set to allow nodes to exceed the '
+                     'capacity of a single power supply')
+        }
+        ovs = powercfg[1]
+        if ovs == 1:
+            ovs = 'Enable'
+        elif ovs == 0:
+            ovs = 'Disable'
+        settings['power_oversubscription'] = {
+            'default': 'Enable',
+            'value': ovs,
+            'possible': ['Enable', 'Disable'],
+            'help': ('In redundant power configuration, permit the power '
+                     'budget to exceed the capacity of a single power supply '
+                     'so long as both power supplies are functioning. This '
+                     'excess is limited to an amount that the remaining power '
+                     'supply can sustain for a brief period of time in the '
+                     'event of losing the other. This excess will be removed '
+                     'at the moment a power supply fails so that power '
+                     'delivery is at the sustained capacity of the remaining '
+                     'supplies.')
+        }
+        try:
+            dhcpsendname = self.ipmicmd.xraw_command(0xc, 0x2,
+                                                     data=[1, 0xc5, 0, 0])
+            dhcpsendname = bytearray(dhcpsendname['data'])
+            dhcpsendname = 'Enable' if dhcpsendname[1] == 1 else 'Disable'
+            settings['dhcp_sends_hostname'] = {
+                'value': dhcpsendname,
+                'help': ('Have the device send  hostname as part of its '
+                         'DHCP client requests in option 12'),
+                'possible': ['Enable', 'Disable']
+            }
+            dhcpsendvci = self.ipmicmd.xraw_command(0xc, 0x2,
+                                                    data=[1, 0xc6, 0, 0])
+            dhcpsendvci = bytearray(dhcpsendvci['data'])
+            dhcpsendvci = 'Enable' if dhcpsendvci[1] == 1 else 'Disable'
+            settings['dhcp_sends_vendor_class_identifier'] = {
+                'value': dhcpsendvci,
+                'possible': ['Enable', 'Disable'],
+                'help': ('Have the device send vendor class identifier '
+                         'as part of its DHCP requests in option 60')
+            }
+        except Exception:
+            pass
+        return settings
+
+    def set_bmc_configuration(self, changeset):
+        rules = []
+        powercfg = [None, None]
+        sendhost = None
+        sendvci = None
+        for key in changeset:
+            if not key:
+                raise pygexc.InvalidParameterValue('Empty key is invalid')
+            if isinstance(changeset[key], six.string_types):
+                changeset[key] = {'value': changeset[key]}
+            for rule in self.rulemap:
+                if fnmatch.fnmatch(rule, key.lower()):
+                    rules.append('{0}:{1}'.format(
+                        self.rulemap[rule], changeset[key]['value']))
+            if fnmatch.fnmatch('power_redundancy', key.lower()):
+                if 'n+1'.startswith(changeset[key]['value'].lower()):
+                    powercfg[0] = 1
+                elif 'disable'.startswith(changeset[key]['value'].lower()):
+                    powercfg[0] = 0
+            if fnmatch.fnmatch('power_oversubscription', key.lower()):
+                ovs = changeset[key]['value']
+                ovs = stringtoboolean(changeset[key]['value'],
+                                      'power_oversubscription')
+                powercfg[1] = 1 if ovs else 0
+            if fnmatch.fnmatch('dhcp_sends_hostname', key.lower()):
+                sendhost = changeset[key]['value']
+                sendhost = stringtoboolean(changeset[key]['value'],
+                                           'dhcp_sends_hostname')
+            if fnmatch.fnmatch(
+                    'dhcp_sends_vendor_class_identifier', key.lower()):
+                sendvci = changeset[key]['value']
+                sendvci = stringtoboolean(
+                    changeset[key]['value'],
+                    'dhcp_sends_vendor_class_identifier')
+            if fnmatch.fnmatch('fanspeed', key.lower()):
+                for mode in self.fanmodes:
+                    byteval = mode
+                    mode = self.fanmodes[mode]
+                    if changeset[key]['value'].lower() == mode.lower():
+                        self.ipmicmd.xraw_command(
+                            0x32, 0x9b, data=[byteval])
+                        break
+                else:
+                    raise pygexc.InvalidParameterValue(
+                        '{0} not a valid mode for fanspeed'.format(
+                            changeset[key]['value']))
+        if rules:
+            rules = 'set={0}'.format(','.join(rules))
+            self.wc.request('POST', '/data', rules)
+            self.wc.getresponse().read()
+        if powercfg != [None, None]:
+            if None in powercfg:
+                currcfg = self.ipmicmd.xraw_command(0x32, 0xa2)
+                currcfg = bytearray(currcfg['data'])
+                if powercfg[0] is None:
+                    powercfg[0] = currcfg[0]
+                if powercfg[1] is None:
+                    powercfg[1] = currcfg[1]
+            self.ipmicmd.xraw_command(0x32, 0xa3, data=powercfg)
+        if sendhost is not None:
+            sendhost = 1 if sendhost else 0
+            self.ipmicmd.xraw_command(0xc, 1, data=[1, 0xc5, sendhost])
+        if sendvci is not None:
+            sendvci = 1 if sendvci else 0
+            self.ipmicmd.xraw_command(0xc, 1, data=[1, 0xc6, sendvci])
+
     def set_user_priv(self, uid, priv):
         if priv.lower() == 'administrator':
             rsp = self.ipmicmd.xraw_command(netfn=6, command=0x46, data=(uid,))
@@ -259,6 +452,7 @@ class SMMClient(object):
                 'POST', '/data', b'set=user({0},1,{1},511,,4,15,0)'.format(
                     uid, username))
             rsp = self.wc.getresponse()
+            rsp.read()
 
     def reseat_bay(self, bay):
         self.ipmicmd.xraw_command(netfn=0x32, command=0xa4,
@@ -274,7 +468,7 @@ class SMMClient(object):
         rsp['data'] = b'\x01'
         initpct = 1.0
         if progress:
-                progress({'phase': 'initializing', 'progress': initpct})
+            progress({'phase': 'initializing', 'progress': initpct})
         while bytearray(rsp['data'])[0] != 0:
             ipmisession.Session.pause(3)
             initpct += 3.0
@@ -295,6 +489,7 @@ class SMMClient(object):
                           'progress': 100 * self.wc.get_download_progress()})
         if progress:
             progress({'phase': 'complete'})
+        return savefile
 
     def process_fru(self, fru):
         # TODO(jjohnson2): can also get EIOM, SMM, and riser data if warranted
@@ -310,8 +505,12 @@ class SMMClient(object):
         cv = self.ipmicmd.certverify
         wc = webclient.SecureHTTPConnection(self.smm, 443, verifycallback=cv)
         wc.connect()
-        loginform = urllib.urlencode({'user': self.username,
-                                      'password': self.password})
+        loginform = urlencode(
+            {
+                'user': self.username,
+                'password': self.password
+            }
+        )
         wc.request('POST', '/data/login', loginform)
         rsp = wc.getresponse()
         if rsp.status != 200:
