@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 IBM Corporation
 # Copyright 2015-2017 Lenovo
 #
@@ -14,7 +12,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# This represents the low layer message framing portion of IPMI
+
+"""This represents the low layer message framing portion of IPMI"""
 
 import collections
 import hashlib
@@ -27,14 +26,15 @@ import socket
 import struct
 import threading
 
-
 from cryptography.hazmat.backends import default_backend
-from cryptography.hazmat.primitives.ciphers import algorithms, Cipher, modes
-
+from cryptography.hazmat.primitives.ciphers import algorithms
+from cryptography.hazmat.primitives.ciphers import Cipher
+from cryptography.hazmat.primitives.ciphers import modes
 import pyghmi.exceptions as exc
 from pyghmi.ipmi.private import constants
 from pyghmi.ipmi.private import util
-from pyghmi.ipmi.private.util import get_ipmi_error, _monotonic_time
+from pyghmi.ipmi.private.util import _monotonic_time
+from pyghmi.ipmi.private.util import get_ipmi_error
 
 
 KEEPALIVE_SESSIONS = threading.RLock()
@@ -107,7 +107,7 @@ def define_worker():
                 if timeout < 0:
                     timeout = 0
                 selectdeadline = _monotonic_time() + timeout
-                tmplist, _, _ = select.select(iosockets, (), (), timeout)
+                select.select(iosockets, (), (), timeout)
                 # pessimistically move out the deadline
                 # doing it this early (before ioqueue is evaluated)
                 # this avoids other threads making a bad assumption
@@ -206,8 +206,8 @@ def _io_graball(mysockets, iowaiters):
             myport = mysocket.getsockname()[1]
             rdata = rdata + (mysocket,)
             relsession = None
-            if (rdata[1] in Session.bmc_handlers and
-                    myport in Session.bmc_handlers[rdata[1]]):
+            if (rdata[1] in Session.bmc_handlers
+                    and myport in Session.bmc_handlers[rdata[1]]):
                 # session data
                 rdata = rdata + (True,)
                 relsession = Session.bmc_handlers[rdata[1]][myport]
@@ -250,8 +250,9 @@ def _poller(timeout=0):
 
 
 def _aespad(data):
-    """ipmi demands a certain pad scheme,
-    per table 13-20 AES-CBC encrypted payload fields.
+    """ipmi demands a certain pad scheme, per table 13-20 AES-CBC encrypted
+
+    payload fields.
     """
     currlen = len(data) + 1  # need to count the pad length field as well
     neededpad = currlen % 16
@@ -390,6 +391,7 @@ class Session(object):
             iothreadwaiters.append(initevt)
             _IOWorker = define_worker()
             iothread = _IOWorker()
+            iothread.daemon = True
             iothread.start()
             initevt.wait()
         elif not iothreadready:
@@ -399,9 +401,7 @@ class Session(object):
         return tmpsocket
 
     def _sync_login(self, response):
-        """Handle synchronous callers in liue of
-        a client-provided callback.
-        """
+        """Handle synchronous callers in lieu of a client-provided callback"""
         # Be a stub, the __init__ will catch and respond to ensure response
         # is given in the same thread as was called
         return
@@ -436,13 +436,15 @@ class Session(object):
             if sockaddr in cls.bmc_handlers:
                 for portself in list(dictitems(cls.bmc_handlers[sockaddr])):
                     self = portself[1]
-                    if not ((self.logged or self.logging) and
-                            cls._is_session_valid(self)):
+                    if not ((self.logged or self.logging)
+                            and cls._is_session_valid(self)):
                         # we have encountered a leftover broken session
                         del cls.bmc_handlers[sockaddr][portself[0]]
                         continue
-                    if (self.bmc == bmc and self.userid == userid and
-                            self.password == password and self.kgo == kg):
+                    if (self.bmc == bmc
+                            and self.userid == userid
+                            and self.password == password
+                            and self.kgo == kg):
                         trueself = self
                         break
                     # ok, the candidate seems to be working, but does not match
@@ -494,6 +496,7 @@ class Session(object):
         else:
             self.privlevel = 4
             self.autopriv = True
+        self.logoutexpiry = None
         self.autokeepalive = keepalive
         self.maxtimeout = 3  # be aggressive about giving up on initial packet
         self.incommand = False
@@ -560,7 +563,7 @@ class Session(object):
                                            self.kgo)]
         except KeyError:
             pass
-        self.logout()
+        self.logout(False)
         self.logging = False
         self.errormsg = error
         if not self.broken:
@@ -596,6 +599,8 @@ class Session(object):
         self.authtype = 0
         self.ipmiversion = 1.5
         self.timeout = initialtimeout + (0.5 * random.random())
+        self.logoutexpiry = _monotonic_time() + self._getmaxtimeout()
+        self.rqlun = 0
         self.seqlun = 0
         # NOTE(jbjohnso): per IPMI table 5-4, software ids in the ipmi spec may
         #                 be 0x81 through 0x8d.  We'll stick with 0x81 for now,
@@ -621,14 +626,17 @@ class Session(object):
 
     def _make_bridge_request_msg(self, channel, netfn, command):
         """This function generate message for bridge request. It is a
+
         part of ipmi payload.
         """
+        # NOTE(puwen): need to pay attention for this function because the
+        # structure does not seem to match the specifications.
         head = bytearray((constants.IPMI_BMC_ADDRESS,
                           constants.netfn_codes['application'] << 2))
         check_sum = _checksum(*head)
         # NOTE(fengqian): according IPMI Figure 14-11, rqSWID is set to 81h
-        boday = bytearray((0x81, self.seqlun, constants.IPMI_SEND_MESSAGE_CMD,
-                           0x40 | channel))
+        boday = bytearray((0x81, (self.seqlun << 2) | self.rqlun,
+                           constants.IPMI_SEND_MESSAGE_CMD, 0x40 | channel))
         # NOTE(fengqian): Track request
         self._add_request_entry((constants.netfn_codes['application'] + 1,
                                  self.seqlun, constants.IPMI_SEND_MESSAGE_CMD))
@@ -636,6 +644,7 @@ class Session(object):
 
     def _add_request_entry(self, entry=()):
         """This function record the request with netfn, sequence number and
+
         command, which will be used in parse_ipmi_payload.
         :param entry: a set of netfn, sequence number and command.
         """
@@ -649,8 +658,10 @@ class Session(object):
         if self._lookup_request_entry(entry):
             self.request_entry.remove(entry)
 
-    def _make_ipmi_payload(self, netfn, command, bridge_request=None, data=()):
+    def _make_ipmi_payload(self, netfn, command, bridge_request=None, data=(),
+                           rslun=0):
         """This function generates the core ipmi payload that would be
+
         applicable for any channel (including KCS)
         """
         bridge_msg = []
@@ -661,13 +672,14 @@ class Session(object):
         # we are always the requestor for now
         seqincrement = 7  # IPMI spec forbids gaps bigger then 7 in seq number.
         # Risk the taboo rather than violate the rules
-        while (not self.servermode and
-                (netfn, command, self.seqlun) in self.tabooseq and
-               self.tabooseq[(netfn, command, self.seqlun)] and seqincrement):
+        while (not self.servermode
+               and (netfn, command, self.seqlun) in self.tabooseq
+               and self.tabooseq[(netfn, command, self.seqlun)]
+               and seqincrement):
             self.tabooseq[(self.expectednetfn, command, self.seqlun)] -= 1
             # Allow taboo to eventually expire after a few rounds
-            self.seqlun += 4  # the last two bits are lun, so add 4 to add 1
-            self.seqlun &= 0xff  # we only have one byte, wrap when exceeded
+            self.seqlun += 1  # the last two bits are lun, so add 4 to add 1
+            self.seqlun &= 0x3f  # we only have one byte, wrap when exceeded
             seqincrement -= 1
 
         if bridge_request:
@@ -686,9 +698,9 @@ class Session(object):
         # figure 13-4, first two bytes are rsaddr and
         # netfn, for non-bridge request, rsaddr is always 0x20 since we are
         # addressing BMC while rsaddr is specified forbridge request
-        header = bytearray((rsaddr, netfn << 2))
-
-        reqbody = bytearray((rqaddr, self.seqlun, command)) + data
+        header = bytearray((rsaddr, (netfn << 2) | rslun))
+        reqbody = bytearray(
+            (rqaddr, (self.seqlun << 2) | self.rqlun, command)) + data
         headsum = bytearray((_checksum(*header),))
         bodysum = bytearray((_checksum(*reqbody),))
         payload = header + headsum + reqbody + bodysum
@@ -738,8 +750,8 @@ class Session(object):
                 waiter({'success': True})
             self.process_pktqueue()
             with util.protect(WAITING_SESSIONS):
-                if (self in self.waiting_sessions and
-                        self.expiration < _monotonic_time()):
+                if (self in self.waiting_sessions
+                        and self.expiration < _monotonic_time()):
                     self.waiting_sessions.pop(self, None)
                     self._timedout()
 
@@ -751,8 +763,12 @@ class Session(object):
                     retry=True,
                     delay_xmit=None,
                     timeout=None,
-                    callback=None):
+                    callback=None,
+                    rslun=0):
         if not self.logged:
+            if (self.logoutexpiry is not None
+                    and _monotonic_time() > self.logoutexpiry):
+                self._mark_broken()
             raise exc.IpmiException('Session no longer connected')
         self._cmdwait()
         if not self.logged:
@@ -766,7 +782,7 @@ class Session(object):
         self._send_ipmi_net_payload(netfn, command, data,
                                     bridge_request=bridge_request,
                                     retry=retry, delay_xmit=delay_xmit,
-                                    timeout=timeout)
+                                    timeout=timeout, rslun=rslun)
 
         if retry:  # in retry case, let the retry timers indicate wait time
             timeout = None
@@ -792,7 +808,8 @@ class Session(object):
 
     def _send_ipmi_net_payload(self, netfn=None, command=None, data=(), code=0,
                                bridge_request=None,
-                               retry=None, delay_xmit=None, timeout=None):
+                               retry=None, delay_xmit=None, timeout=None,
+                               rslun=0):
         if retry is None:
             retry = not self.servermode
         if self.servermode:
@@ -804,7 +821,7 @@ class Session(object):
         else:
             data = bytearray(data)
         ipmipayload = self._make_ipmi_payload(netfn, command, bridge_request,
-                                              data)
+                                              data, rslun)
         payload_type = constants.payload_types['ipmi']
         self.send_payload(payload=ipmipayload, payload_type=payload_type,
                           retry=retry, delay_xmit=delay_xmit, timeout=timeout)
@@ -906,16 +923,16 @@ class Session(object):
                 message.append(neededpad)
                 message.append(7)  # reserved, 7 is the required value for the
                 # specification followed
-                message += hmac.new(self.k1,
-                                    bytes(message[4:]),
-                                    self.currhashlib).digest()[:self.currhashlen]
+                message += hmac.new(
+                    self.k1, bytes(message[4:]),
+                    self.currhashlib).digest()[:self.currhashlen]
                 # per RFC2404 truncates to 96 bits
         self.netpacket = message
         # advance idle timer since we don't need keepalive while sending
         # packets out naturally
         with util.protect(KEEPALIVE_SESSIONS):
-            if (self in Session.keepalive_sessions and not needskeepalive and
-                    not self._customkeepalives):
+            if (self in Session.keepalive_sessions and not needskeepalive
+                    and not self._customkeepalives):
                 Session.keepalive_sessions[self]['timeout'] = \
                     _monotonic_time() + MAX_IDLE - (random.random() * 4.9)
             self._xmit_packet(retry, delay_xmit=delay_xmit, timeout=timeout)
@@ -933,7 +950,7 @@ class Session(object):
             raise exc.IpmiException("Password is too long for ipmi 1.5")
         password += '\x00' * padneeded
         if checkremotecode:
-            seqbytes = struct.pack("<I", self.remsequencenumber)
+            seqbytes = struct.pack("<I", self.remseqnumber)
         else:
             seqbytes = struct.pack("<I", self.sequencenumber)
         sessdata = struct.pack("<I", self.sessionid)
@@ -1003,6 +1020,7 @@ class Session(object):
 
     def _req_priv_level(self):
         self.logged = 1
+        self.logoutexpiry = None
         response = self.raw_command(netfn=0x6, command=0x3b,
                                     data=[self.privlevel])
         if response['code']:
@@ -1134,8 +1152,8 @@ class Session(object):
                     if parms['timeout'] <= curtime:
                         timeout = 0  # exit after one guaranteed pass
                         break
-                    if (timeout is not None and
-                            timeout < parms['timeout'] - curtime):
+                    if (timeout is not None
+                            and timeout < parms['timeout'] - curtime):
                         continue  # timeout smaller than the current session
                         # needs
                     timeout = parms['timeout'] - curtime  # set new timeout
@@ -1145,8 +1163,8 @@ class Session(object):
                     if parms['timeout'] <= curtime:
                         timeout = 0
                         break
-                    if (timeout is not None and
-                            timeout < parms['timeout'] - curtime):
+                    if (timeout is not None
+                            and timeout < parms['timeout'] - curtime):
                         continue
                     timeout = parms['timeout'] - curtime
         # If the loop above found no sessions wanting *and* the caller had no
@@ -1239,8 +1257,8 @@ class Session(object):
         return _keptalive
 
     def _keepalive(self):
-        """Performs a keepalive to avoid idle disconnect
-        """
+        """Performs a keepalive to avoid idle disconnect"""
+
         try:
             keptalive = False
             if self._customkeepalives:
@@ -1267,7 +1285,7 @@ class Session(object):
                     return
                 if self.autokeepalive:
                     self.raw_command(netfn=6, command=1,
-                                    callback=self._keepalive_wrapper(None))
+                                     callback=self._keepalive_wrapper(None))
                 else:
                     self.logout()
         except exc.IpmiException:
@@ -1290,17 +1308,17 @@ class Session(object):
     def _handle_ipmi_packet(self, data, sockaddr=None, qent=None):
         if self.sockaddr is None and sockaddr is not None:
             self.sockaddr = sockaddr
-        elif (self.sockaddr is not None and
-              sockaddr is not None and
-              self.sockaddr != sockaddr):
+        elif (self.sockaddr is not None
+              and sockaddr is not None
+              and self.sockaddr != sockaddr):
             return  # here, we might have sent an ipv4 and ipv6 packet to kick
             # things off ignore the second reply since we have one
             # satisfactory answer
         if data[4] in (0, 2):  # This is an ipmi 1.5 paylod
-            remsequencenumber = struct.unpack('<I', bytes(data[5:9]))[0]
+            remseqnumber = struct.unpack('<I', bytes(data[5:9]))[0]
             remsessid = struct.unpack("<I", bytes(data[9:13]))[0]
-            if (remsequencenumber == 0 and remsessid == 0 and
-                    qent[2] in Session.bmc_handlers):
+            if (remseqnumber == 0 and remsessid == 0
+                    and qent[2] in Session.bmc_handlers):
                 # So a new ipmi client happens to get a previously seen and
                 # still active UDP source port.  Clear ourselves out and punt
                 # to IpmiServer
@@ -1309,10 +1327,10 @@ class Session(object):
                 iserver.pktqueue.append(qent)
                 iserver.process_pktqueue()
                 return
-            if (hasattr(self, 'remsequencenumber') and
-                    remsequencenumber < self.remsequencenumber):
+            if (hasattr(self, 'remseqnumber')
+                    and remseqnumber < self.remseqnumber):
                 return -5  # remote sequence number is too low, reject it
-            self.remsequencenumber = remsequencenumber
+            self.remseqnumber = remseqnumber
             if data[4] != self.authtype:
                 # BMC responded with mismatch authtype, for
                 # mutual authentication reject it. If this causes
@@ -1379,16 +1397,17 @@ class Session(object):
             if self.k1 is None:  # we are in no shape to process a packet now
                 return
             expectedauthcode = hmac.new(
-                self.k1, data[4:-self.currhashlen], self.currhashlib).digest()[:self.currhashlen]
+                self.k1, data[4:-self.currhashlen],
+                self.currhashlib).digest()[:self.currhashlen]
             if authcode != expectedauthcode:
                 return  # BMC failed to assure integrity to us, drop it
             sid = struct.unpack("<I", bytes(data[6:10]))[0]
             if sid != self.localsid:  # session id mismatch, drop it
                 return
             remseqnumber = struct.unpack("<I", bytes(data[10:14]))[0]
-            if (hasattr(self, 'remseqnumber') and
-                (remseqnumber < self.remseqnumber) and
-                    (self.remseqnumber != 0xffffffff)):
+            if (hasattr(self, 'remseqnumber')
+                    and (remseqnumber < self.remseqnumber)
+                    and (self.remseqnumber != 0xffffffff)):
                 return
             self.remseqnumber = remseqnumber
             psize = data[14] + (data[15] << 8)
@@ -1504,11 +1523,13 @@ class Session(object):
         self.remoterandombytes = bytes(data[8:24])
         self.remoteguid = bytes(data[24:40])
         userlen = len(self.userid)
-        hmacdata = struct.pack("<II", localsid, self.pendingsessionid) +\
-            self.randombytes + self.remoterandombytes + self.remoteguid +\
-            struct.pack("2B", self.nameonly | self.privlevel, userlen) +\
-            self.userid
-        expectedhash = hmac.new(self.password, hmacdata, self.currhashlib).digest()
+        hmacdata = (struct.pack("<II", localsid, self.pendingsessionid)
+                    + self.randombytes + self.remoterandombytes
+                    + self.remoteguid + struct.pack(
+                    "2B", self.nameonly | self.privlevel, userlen)
+                    + self.userid)
+        expectedhash = hmac.new(self.password, hmacdata,
+                                self.currhashlib).digest()
         hashlen = len(expectedhash)
         givenhash = struct.pack("%dB" % hashlen, *data[40:hashlen + 40])
         if givenhash != expectedhash:
@@ -1518,10 +1539,10 @@ class Session(object):
         # We have now validated that the BMC and client agree on password, time
         # to store the keys
         self.sik = hmac.new(self.kg,
-                            self.randombytes + self.remoterandombytes +
-                            struct.pack("2B", self.nameonly | self.privlevel,
-                                        userlen) +
-                            self.userid, self.currhashlib).digest()
+                            self.randombytes + self.remoterandombytes
+                            + struct.pack(
+                                "2B", self.nameonly | self.privlevel, userlen)
+                            + self.userid, self.currhashlib).digest()
         self.k1 = hmac.new(self.sik, b'\x01' * 20, self.currhashlib).digest()
         self.k2 = hmac.new(self.sik, b'\x02' * 20, self.currhashlib).digest()
         self.aeskey = self.k2[0:16]
@@ -1575,8 +1596,8 @@ class Session(object):
         hmacdata = self.randombytes +\
             struct.pack("<I", self.pendingsessionid) +\
             self.remoteguid
-        expectedauthcode = hmac.new(self.sik, hmacdata,
-                                    self.currhashlib).digest()[:self.currhashlen]
+        expectedauthcode = hmac.new(
+            self.sik, hmacdata, self.currhashlib).digest()[:self.currhashlen]
         aclen = len(expectedauthcode)
         authcode = struct.pack("%dB" % aclen, *data[8:aclen + 8])
         if authcode != expectedauthcode:
@@ -1605,13 +1626,15 @@ class Session(object):
             # drop it and carry about our business.
             return
         if self.servermode:
-            self.seqlun = payload[4]
+            self.seqlun = payload[4] >> 2
             self.clientaddr = payload[3]
             self.clientnetfn = (payload[1] >> 2) + 1
+            self.rqlun = payload[4] & 0b11
             self.clientcommand = payload[5]
             self._parse_payload(payload)
             return
-        entry = (payload[1] >> 2, payload[4], payload[5])
+        # payload[4] is rqSeq + rsLUN, we only need the rqSeq
+        entry = (payload[1] >> 2, payload[4] >> 2, payload[5])
         if self._lookup_request_entry(entry):
             self._remove_request_entry(entry)
 
@@ -1620,8 +1643,8 @@ class Session(object):
             # second response is the real response.
             # If the message is send crrectly, we will discard the first
             # response or else error message will be parsed and return.
-            if ((entry[0] in [0x06, 0x07]) and (entry[2] == 0x34) and
-                    (payload[-2] == 0x0)):
+            if ((entry[0] in [0x06, 0x07]) and (entry[2] == 0x34)
+                    and (payload[-2] == 0x0)):
                 return -1
             else:
                 self._parse_payload(payload)
@@ -1647,8 +1670,8 @@ class Session(object):
         self.expectednetfn = 0x1ff
         self.expectedcmd = 0x1ff
         if not self.servermode:
-            self.seqlun += 4  # prepare seqlun for next transmit
-            self.seqlun &= 0xff  # when overflowing, wrap around
+            self.seqlun += 1  # prepare seqlun for next transmit
+            self.seqlun &= 0x3f  # when overflowing, wrap around
         with util.protect(WAITING_SESSIONS):
             Session.waiting_sessions.pop(self, None)
         # render retry mechanism utterly incapable of
@@ -1698,8 +1721,8 @@ class Session(object):
             # delayed in this case, it's safe to just redo the request
             self.lastpayload = None
             self._open_rmcpplus_request()
-        elif (self.sessioncontext == 'EXPECTINGRAKP2' or
-              self.sessioncontext == 'EXPECTINGRAKP4'):
+        elif (self.sessioncontext == 'EXPECTINGRAKP2'
+              or self.sessioncontext == 'EXPECTINGRAKP4'):
             # If we can't be sure which RAKP was dropped or if RAKP3/4 was just
             # delayed, the most reliable thing to do is rewind and start over
             # bmcs do not take kindly to receiving RAKP1 or RAKP3 twice
@@ -1770,11 +1793,14 @@ class Session(object):
                     self.expiration = self.timeout + _monotonic_time()
                 Session.waiting_sessions[self]['timeout'] = self.expiration
 
-    def logout(self):
+    def logout(self, sessionok=True):
         if not self.logged:
             return {'success': True}
         if self.cleaningup:
             self.nowait = True
+        if self.sol_handler:
+            self.raw_command(netfn=6, command=0x49, data=(1, 1, 0, 0, 0, 0),
+                             retry=sessionok)
         self.raw_command(command=0x3c,
                          netfn=6,
                          data=struct.unpack("4B",
@@ -1804,8 +1830,8 @@ class Session(object):
             # replace this one
             myport = self.socket.getsockname()[1]
             for sockaddr in self.allsockaddrs:
-                if (sockaddr in Session.bmc_handlers and
-                        myport in Session.bmc_handlers[sockaddr]):
+                if (sockaddr in Session.bmc_handlers
+                        and myport in Session.bmc_handlers[sockaddr]):
                     del Session.bmc_handlers[sockaddr][myport]
                     if Session.bmc_handlers[sockaddr] == {}:
                         del Session.bmc_handlers[sockaddr]
