@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # coding=utf8
-
 # Copyright 2014 IBM Corporation
 # Copyright 2015 Lenovo
 #
@@ -16,28 +14,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# This module provides access to SDR offered by a BMC
-# This data is common between 'sensors' and 'inventory' modules since SDR
-# is both used to enumerate sensors for sensor commands and FRU ids for FRU
-# commands
+"""This module provides access to SDR offered by a BMC
 
-# For now, we will not offer persistent SDR caching as we do in xCAT's IPMI
-# code.  Will see if it is adequate to advocate for high object reuse in a
-# persistent process for the moment.
+This data is common between 'sensors' and 'inventory' modules since SDR
+is both used to enumerate sensors for sensor commands and FRU ids for FRU
+commands
 
-# Focus is at least initially on the aspects that make the most sense for a
-# remote client to care about.  For example, smbus information is being
-# skipped for now
+For now, we will not offer persistent SDR caching as we do in xCAT's IPMI
+code.  Will see if it is adequate to advocate for high object reuse in a
+persistent process for the moment.
+
+Focus is at least initially on the aspects that make the most sense for a
+remote client to care about.  For example, smbus information is being
+skipped for now
+"""
 
 import math
 import os
-import pyghmi.constants as const
-import pyghmi.exceptions as exc
-import pyghmi.ipmi.private.constants as ipmiconst
 import random
 import string
 import struct
 import weakref
+
+import pyghmi.constants as const
+import pyghmi.exceptions as exc
+import pyghmi.ipmi.private.constants as ipmiconst
+import six
+
 try:
     import cPickle as pickle
 except ImportError:
@@ -49,6 +52,7 @@ TYPE_FRU = 2
 
 shared_sdrs = {}
 
+
 def ones_complement(value, bits):
     # utility function to help with the large amount of 2s
     # complement prevalent in ipmi spec
@@ -345,6 +349,7 @@ class SDREntry(object):
         # event only, compact and full are very similar
         # this function handles the common aspects of compact and full
         # offsets from spec, minus 6
+        self.sensor_lun = entry[1] & 0x03
         self.sensor_number = entry[2]
         self.entity = ipmiconst.entity_ids.get(
             entry[3], 'Unknown entity {0}'.format(entry[3]))
@@ -389,6 +394,13 @@ class SDREntry(object):
         self.percent = ''
         if entry[15] & 1 == 1:
             self.percent = '% '
+        if self.sensor_type_number == 0xb:
+            if self.unit_mod == '':
+                if entry[16] == 6:
+                    self.sensor_type = 'Power'
+            elif self.unit_mod == ' ':
+                if entry[16] == 6 and entry[17] in (22, 23, 24):
+                    self.sensor_type = 'Energy'
         self.baseunit = unit_types[entry[16]]
         self.modunit = unit_types[entry[17]]
         self.unit_suffix = self.percent + self.baseunit + self.unit_mod + \
@@ -439,7 +451,7 @@ class SDREntry(object):
             'type': self.sensor_type,
             'id': self.sensor_number,
         }
-        if reading[1] & 0b100000:
+        if reading[1] & 0b100000 or not reading[1] & 0b1000000:
             output['unavailable'] = 1
             return SensorReading(output, self.unit_suffix)
         if self.numeric_format == 2:
@@ -528,8 +540,8 @@ class SDREntry(object):
             self._set_tmp_formula(value)
             linearization = 0
         # time to compute the pre-linearization value.
-        decoded = float((value * self.m + self.b) *
-                        (10 ** self.resultexponent))
+        decoded = float((value * self.m + self.b)
+                        * (10 ** self.resultexponent))
         if linearization == 0:
             return decoded
         elif linearization == 1:
@@ -553,18 +565,15 @@ class SDREntry(object):
         elif linearization == 10:
             return math.sqrt(decoded)
         elif linearization == 11:
-            return decoded ** (1.0/3)
+            return decoded ** (1.0 / 3)
         else:
             raise NotImplementedError
 
     def decode_formula(self, entry):
-        self.m = \
-            twos_complement(entry[0] + ((entry[1] & 0b11000000) << 2), 10)
+        self.m = twos_complement(entry[0] + ((entry[1] & 0b11000000) << 2), 10)
         self.tolerance = entry[1] & 0b111111
-        self.b = \
-            twos_complement(entry[2] + ((entry[3] & 0b11000000) << 2), 10)
-        self.accuracy = (entry[3] & 0b111111) + \
-            (entry[4] & 0b11110000) << 2
+        self.b = twos_complement(entry[2] + ((entry[3] & 0b11000000) << 2), 10)
+        self.accuracy = (entry[3] & 0b111111) + (entry[4] & 0b11110000) << 2
         self.accuracyexp = (entry[4] & 0b1100) >> 2
         self.direction = entry[4] & 0b11
         # 0 = n/a, 1 = input, 2 = output
@@ -580,7 +589,8 @@ class SDREntry(object):
             return ""
         if ipmitype == 0:  # Unicode per 43.15 in ipmi 2.0 spec
             # the spec is not specific about encoding, assuming utf8
-            return unicode(struct.pack("%dB" % len(data), *data), "utf_8")
+            return six.text_type(struct.pack("%dB" % len(data), *data),
+                                 "utf_8")
         elif ipmitype == 1:  # BCD '+'
             tmpl = "%02X" * len(data)
             tstr = tmpl % tuple(data)
@@ -592,14 +602,17 @@ class SDREntry(object):
             tstr = ""
             while len(data) >= 3:  # the packing only works with 3 byte chunks
                 tstr += chr((data[0] & 0b111111) + 0x20)
-                tstr += chr(((data[1] & 0b1111) << 2) +
-                            (data[0] >> 6) + 0x20)
-                tstr += chr(((data[2] & 0b11) << 4) +
-                            (data[1] >> 4) + 0x20)
+                tstr += chr(((data[1] & 0b1111) << 2) + (data[0] >> 6) + 0x20)
+                tstr += chr(((data[2] & 0b11) << 4) + (data[1] >> 4) + 0x20)
                 tstr += chr((data[2] >> 2) + 0x20)
+            if not isinstance(tstr, str):
+                tstr = tstr.decode('utf-8')
             return tstr
         elif ipmitype == 3:  # ACSII+LATIN1
-            return struct.pack("%dB" % len(data), *data)
+            ret = struct.pack("%dB" % len(data), *data)
+            if not isinstance(ret, str):
+                ret = ret.decode('utf-8')
+            return ret
 
 
 class SDR(object):
@@ -629,9 +642,9 @@ class SDR(object):
         self.fw_major = rsp['data'][2] & 0b1111111
         self.fw_minor = "%02X" % rsp['data'][3]  # BCD encoding, oddly enough
         self.ipmiversion = rsp['data'][4]  # 51h = 1.5, 02h = 2.0
-        self.mfg_id = rsp['data'][8] << 16 + rsp['data'][7] << 8 + \
+        self.mfg_id = (rsp['data'][8] << 16) + (rsp['data'][7] << 8) + \
             rsp['data'][6]
-        self.prod_id = rsp['data'][10] << 8 + rsp['data'][9]
+        self.prod_id = (rsp['data'][10] << 8) + rsp['data'][9]
         if len(rsp['data']) > 11:
             self.aux_fw = self.decode_aux(rsp['data'][11:15])
         if rsp['data'][1] & 0b10000000 and rsp['data'][5] & 0b10 == 0:
@@ -695,7 +708,7 @@ class SDR(object):
                 self.fw_minor, modtime)
             cachefilename = os.path.join(self.cachedir, cachefilename)
         if cachefilename and os.path.isfile(cachefilename):
-            with open(cachefilename, 'r') as cfile:
+            with open(cachefilename, 'rb') as cfile:
                 csdrs = pickle.load(cfile)
                 for sdrdata in csdrs:
                     self.add_sdr(sdrdata)
@@ -777,7 +790,7 @@ class SDR(object):
         if cachefilename:
             suffix = ''.join(
                 random.choice(string.ascii_lowercase) for _ in range(12))
-            with open(cachefilename + '.' + suffix, 'w') as cfile:
+            with open(cachefilename + '.' + suffix, 'wb') as cfile:
                 pickle.dump(sdrraw, cfile)
             os.rename(cachefilename + '.' + suffix, cachefilename)
 
@@ -789,7 +802,7 @@ class SDR(object):
     def add_sdr(self, sdrbytes):
         newent = SDREntry(sdrbytes, self.ipmicmd)
         if newent.sdrtype == TYPE_SENSOR:
-            id = newent.sensor_number
+            id = '{0}.{1}'.format(newent.sensor_number, newent.sensor_lun)
             if id in self.sensors:
                 self.broken_sensor_ids[id] = True
                 return
@@ -805,20 +818,3 @@ class SDR(object):
         # This is where manufacturers can add their own
         # decode information
         return "".join(hex(x) for x in auxdata)
-
-if __name__ == "__main__":  # test code
-    import os
-    import pyghmi.ipmi.command as ipmicmd
-    import sys
-    password = os.environ['IPMIPASSWORD']
-    bmc = sys.argv[1]
-    user = sys.argv[2]
-    ipmicmd = ipmicmd.Command(bmc=bmc, userid=user, password=password)
-    sdr = SDR(ipmicmd)
-    for number in sdr.get_sensor_numbers():
-        rsp = ipmicmd.raw_command(command=0x2d, netfn=4, data=(number,))
-        if 'error' in rsp:
-            continue
-        reading = sdr.sensors[number].decode_sensor_reading(rsp['data'])
-        if reading is not None:
-            print(repr(reading))
